@@ -167,7 +167,10 @@ module FileOption = struct
           set_file_priority file priority;
 
           if !verbose && !CommonGlobals.is_startup_phase then
-            lprintf_nl "Started download of %s" (file_best_name file);
+            lprintf_nl "Started download of %s, user:group %s:%s"
+              (file_best_name file)
+              file_user.user_name
+              (user2_print_group file_group);
 
           file
       | _ -> assert false
@@ -1241,6 +1244,12 @@ let _ =
     shorten_all_file_filenames !!max_filenames
   );
 
+  option_hook max_upload_slots (fun _ ->
+      if !!max_upload_slots < 3 then
+        max_upload_slots =:= 3;
+      networks_iter (fun n -> network_check_upload_slots n)
+    );
+
   let max_opened_connections_pass = ref 0 in
   option_hook max_opened_connections (fun _ ->
 
