@@ -283,14 +283,28 @@ let user2_filter_files files gui_user =
     (fun file -> user2_can_view_file gui_user (file_owner file) (file_group file)) files in
   newlist
 
+let file_state file =
+  let impl = as_file_impl file in
+  impl.impl_file_state
+
 let user2_num_user_dls user =
+  let file_is_in_download_queue file =
+    match file_state file with
+    | FileNew | FileDownloading | FileQueued | FilePaused | FileDownloaded -> true
+    | FileShared | FileCancelled | FileAborted _ -> false
+  in
   let n = ref 0 in
-  H.iter (fun f -> if file_owner f = user then incr n) files_by_num;
+  H.iter (fun f -> if file_owner f = user && file_is_in_download_queue f then incr n) files_by_num;
   !n
 
 let user2_num_group_dls group =
+  let file_is_in_download_queue file =
+    match file_state file with
+    | FileNew | FileDownloading | FileQueued | FilePaused | FileDownloaded -> true
+    | FileShared | FileCancelled | FileAborted _ -> false
+  in
   let n = ref 0 in
-  H.iter (fun f -> if file_group f = Some group then incr n) files_by_num;
+  H.iter (fun f -> if file_group f = Some group && file_is_in_download_queue f then incr n) files_by_num;
   !n
 
 let set_file_state file state =
@@ -445,10 +459,6 @@ let file_find num =
     dummy_file_impl   with impl_file_num = num
   })
 
-let file_state file =
-  let impl = as_file_impl file in
-  impl.impl_file_state
-
 let file_add_source (file : file) c =
   client_must_update c;
   CommonEvent.add_event (File_add_source_event (file,c))
@@ -714,7 +724,7 @@ let file_print file o =
         ("", "sr", Printf.sprintf "%d" (file_priority file)) ];
 
       Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-      if user2_allow_file_admin file o.conn_user.ui_user then
+      if user2_is_admin o.conn_user.ui_user then
         let optionlist = ref "" in
         user2_users_iter (fun user ->
 	  if user <> (file_owner file) then
