@@ -98,6 +98,8 @@ let networks_string = ref ""
 
 let patches_string = ref ""
 
+let is_startup_phase = ref true
+
 let version () =
   Printf.sprintf "MLNet %s: Multi-Network p2p client (%s)"
     Autoconf.current_version !networks_string
@@ -117,7 +119,7 @@ let shorten str limit =
     with e -> slen
   in
   let diff_len_utf8_ascii = slen - len in
-  let max_len = maxi limit 10 in
+  let max_len = max limit 10 in
   if len > max_len then
     let prefix = String.sub name 0 (max_len - 7 + diff_len_utf8_ascii) in
     let suffix = String.sub name (len - 4 + diff_len_utf8_ascii) 4 in
@@ -147,11 +149,6 @@ let find_port server_name bind_addr port_option handler =
     iter !!port_option
   else None
 
-let one_day = 3600. *. 24.
-let half_day = one_day /. 2.
-
-let minutes25 = 25 * 60
-
 let new_connection_control () = {
     control_last_ok = 0;
     control_state = 0;
@@ -160,7 +157,7 @@ let new_connection_control () = {
   }
 
 let new_connection_control_recent_ok () = {
-    control_last_ok = last_time () - minutes25;
+    control_last_ok = last_time () - (Date.minute_in_secs * 25);
     control_state = 0;
     control_last_try = 0;
     control_min_reask = !!min_reask_delay;
@@ -177,7 +174,7 @@ let connection_failed cc =
   cc.control_state <- cc.control_state + 1
 
 let connection_next_try cc =
-  cc.control_last_try + mini (cc.control_min_reask * cc.control_state)
+  cc.control_last_try + min (cc.control_min_reask * cc.control_state)
   cc.control_min_reask
 
 let connection_can_try cc =
@@ -238,7 +235,7 @@ let _ =
 	float_of_int (if !!max_hard_upload_rate = 0 then 
 	   10000 * 1024
          else 
-	   maxi (!!max_hard_upload_rate * 1024) 1024) *. 0.90;
+	   max (!!max_hard_upload_rate * 1024) 1024) *. 0.90;
   );
   option_hook max_hard_download_rate (fun _ ->
       check_ul_dl_ratio ();
@@ -249,8 +246,6 @@ let udp_write_controler = UdpSocket.new_bandwidth_controler upload_control
 
 let udp_read_controler = UdpSocket.new_bandwidth_controler download_control
 
-let gui_server_sock = ref (None : TcpServerSocket.t option)
-
 let pid = Unix.getpid ()
 
 let do_at_exit f =
@@ -263,6 +258,7 @@ let exit_properly n = Pervasives.exit n
 let user_socks = ref ([] : TcpBufferedSocket.t list)
 let dialog_history = ref ([] : (int * string * string) list )
 
+exception Incoming_full
 
 let want_and_not andnot f none value =
 (*   lprintf "want_and_not [%s]\n" value; *)
@@ -337,54 +333,13 @@ let rec find_tag name tags =
   (* first GUI have gui_num = 2, since newly created objects have _update = 1 *)
 let gui_counter = ref 2
 
-  (*
-let ip_of_addr addr f =
-  if addr.addr_name <> "" then
-    if addr.addr_age + !!ip_cache_timeout < last_time () then begin
-        Ip.async_ip  addr.addr_name (fun ip ->
-            addr.addr_ip <- ip;
-            addr.addr_age <- last_time ();
-            f ip)
-      end else
-      f addr.addr_ip
-  else
-    f addr.addr_ip
-
-let sync_ip_of_addr addr =
-  if addr.addr_name <> "" then
-    if addr.addr_age + !!ip_cache_timeout < last_time () then begin
-        let ip = Ip.from_name  addr.addr_name in
-        addr.addr_ip <- ip;
-        addr.addr_age <- last_time ();
-        ip
-      end else
-      addr.addr_ip
-  else
-    addr.addr_ip
-
-let new_addr_ip ip = {
-    addr_ip = ip; addr_name = Ip.to_string ip; addr_age = 0;
-  }
-
-let new_addr_name name = {
-    addr_ip = Ip.null; addr_name = name; addr_age = 0
-  }
-
-let string_of_addr addr =
-  if addr.addr_name = "" then Ip.to_string addr.addr_ip else addr.addr_name
-
-let addr_of_string s =
-  let ip = try Ip.of_string s with _ -> Ip.null in
-  if ip <> Ip.null then new_addr_ip ip else new_addr_name s
-
-let addr_is_ip addr = addr.addr_name = ""
-    *)
-
 let upload_counter = ref Int64.zero
 let download_counter = ref Int64.zero
 let nshared_files = ref 0
 let nshared_bytes = ref Int64.zero
 let shared_counter = ref Int64.zero
+let has_upload = ref 0
+let upload_credit = ref 0
 
 let string_of_field t =
   match t with
@@ -398,6 +353,7 @@ let string_of_field t =
   | Field_Completesources -> "completesources"
   | Field_Filename -> "filename"
   | Field_Size -> "size"
+  | Field_Size_Hi -> "size_hi"
   | Field_Uid -> "uid"
   | Field_Bitrate -> "bitrate"
   | Field_Codec -> "codec"
@@ -405,6 +361,7 @@ let string_of_field t =
   | Field_Lastseencomplete -> "lastcompl"
   | Field_Medialength -> "mlen"
   | Field_Mediacodec -> "mediacodec"
+  | Field_KNOWN s -> s
   | Field_UNKNOWN s -> s
 
 let field_of_string t =
@@ -419,6 +376,7 @@ let field_of_string t =
   | "completesources" -> Field_Completesources
   | "filename" -> Field_Filename
   | "size" -> Field_Size
+  | "size_hi" -> Field_Size_Hi
   | "uid" -> Field_Uid
   | "bitrate" -> Field_Bitrate
   | "codec" -> Field_Codec
@@ -426,10 +384,11 @@ let field_of_string t =
   | "lastcompl" -> Field_Lastseencomplete 
   | "mlen" -> Field_Medialength
   | "mediacodec" -> Field_Mediacodec
-  | _ -> Field_UNKNOWN t
+  | _ -> Field_KNOWN t
 
 let escaped_string_of_field tag =
   match tag.tag_name with
+  | Field_KNOWN s -> String.escaped s
   | Field_UNKNOWN s -> String.escaped s
   | t -> string_of_field t
 
@@ -438,6 +397,10 @@ let string_of_tag tag =
   Printf.sprintf "  \"%s\" = %s" (escaped_string_of_field tag)
   (string_of_tag_value tag.tag_value)
 
+let hexstring_of_tag tag =
+  Printf.sprintf "  \"%s\" = %s" (String2.hex_string_of_string (escaped_string_of_field tag))
+  (string_of_tag_value tag.tag_value)
+
 let rec print_tags tags =
   match tags with
     [] -> ()
@@ -481,51 +444,22 @@ let chat_message_fifo = (Fifo.create () : (int * string * int * string * string)
 
 let log_chat_message i num n s =
   Fifo.put chat_message_fifo (last_time(),i,num,n,s);
-  try
+  (try
     Unix2.tryopen_write_gen !messages_log [Open_creat; Open_wronly; Open_append] 
       0o600 (fun oc ->
 	Printf.fprintf oc "%s: %s (%s): %s\n" (Date.simple (BasicSocket.date_of_int (last_time ()))) n i s)
   with e ->
     lprintf_nl "[ERROR] Exception %s while trying to log message to %s"
-      (Printexc2.to_string e) !messages_log;
+      (Printexc2.to_string e) !messages_log);
 
   while (Fifo.length chat_message_fifo) > !!html_mods_max_messages do
     ignore(Fifo.take chat_message_fifo)
   done
+
 let last_message_log = ref 0
 
-
-
 let debug_clients = ref Intset.empty
 
-let default_user = {
-    ui_user_name = CommonUserDb.admin_user;
-    ui_user_searches = [];
-    ui_last_search = None;
-    ui_last_results = [];
-    ui_http_conn = None;
-  }
-
-let ui_users = ref [default_user]
-
-let find_ui_user user =
-  let rec iter list =
-    match list with
-      [] ->
-        let u = {
-            ui_user_name = user;
-            ui_user_searches = [];
-            ui_last_search = None;
-            ui_last_results = [];
-            ui_http_conn = None;
-          } in
-        ui_users := u :: !ui_users;
-        u
-    | u :: tail ->
-        if u.ui_user_name = user then u else iter tail
-  in
-  iter !ui_users
-
 (* control_: means that it is the limited bandwidth, not the unlimited one
   used by the interfaces. tcp_: the full bandwidth (limited+unlimited) *)
 
@@ -617,8 +551,13 @@ let update_link_stats () =
   put time sample short_delay_bandwidth_samples;
   trimto 5 short_delay_bandwidth_samples
 
+let history_step = 5
 let history_size = 720
-let history_h_size = 720
+let history_size_for_h_graph = ref 0 (* history_size * !!html_mods_vd_gfx_h_intervall / 60 *)
+let history_h_step = ref 0 (* 60 * !!html_mods_vd_gfx_h_intervall <= history_size * history_step *)
+let history_h_size = 1440
+let history_timeflag = ref 0.
+let history_h_timeflag = ref 0.
 
 let upload_history = Fifo.create ()
 let download_history = Fifo.create ()
@@ -637,7 +576,7 @@ let download_usage () =
 let update_download_history () =
   Fifo.put download_history (download_usage ());
   let len = ref (Fifo.length download_history) in
-  while !len > history_size do
+  while !len > history_size+1 do
     ignore (Fifo.take download_history);
     decr len
   done
@@ -645,35 +584,46 @@ let update_download_history () =
 let update_upload_history () =
   Fifo.put upload_history (upload_usage ());
   let len = ref (Fifo.length upload_history) in
-  while !len > history_size do
+  while !len > history_size+1 do
     ignore (Fifo.take upload_history);
     decr len
   done
 
 let update_h_download_history () =
-  Fifo.put download_h_history ((List.fold_left (+) 0 (Fifo.to_list download_history)) / ((Fifo.length download_history)));
+  if (!history_h_step = history_size * history_step) or ( (Fifo.length download_history) <= (!history_size_for_h_graph+1)) then 
+    Fifo.put download_h_history ((List.fold_left (+) 0 (Fifo.to_list download_history)) / ((Fifo.length download_history))) 
+  else 
+    Fifo.put download_h_history ((List.fold_left (+) 0 
+		  (snd (List2.cut ((Fifo.length download_history) - !history_size_for_h_graph - 1) (Fifo.to_list download_history)))) 
+      / (!history_size_for_h_graph + 1) );
   let len = ref (Fifo.length download_h_history) in
-  while !len > history_h_size do
+  while !len > history_h_size+1 do
     ignore (Fifo.take download_h_history);
     decr len
   done
 
 let update_h_upload_history () =
-  Fifo.put upload_h_history ((List.fold_left (+) 0 (Fifo.to_list upload_history)) / ((Fifo.length upload_history)));
+	(* Fifo.put upload_h_history ((Fifo.length upload_history) / 720); *)
+  if (!history_h_step = history_size * history_step) or ((Fifo.length upload_history) <= (!history_size_for_h_graph+1)) then 
+    Fifo.put upload_h_history ((List.fold_left (+) 0 (Fifo.to_list upload_history)) / ((Fifo.length upload_history))) 
+  else 
+    Fifo.put upload_h_history ((List.fold_left (+) 0 
+			(snd (List2.cut ((Fifo.length upload_history) - !history_size_for_h_graph - 1) (Fifo.to_list upload_history)))) 
+      / (!history_size_for_h_graph+1) );
   let len = ref (Fifo.length upload_h_history) in
-  while !len > history_h_size do
+  while !len > history_h_size+1 do
     ignore (Fifo.take upload_h_history);
     decr len
   done
 
 let detected_link_capacity link =
-  List.fold_left maxi 0 (Fifo.to_list link)
+  List.fold_left max 0 (Fifo.to_list link)
 
 let detected_uplink_capacity () =
-  List.fold_left maxi 0 (Fifo.to_list upload_history)
+  List.fold_left max 0 (Fifo.to_list upload_history)
 
 let detected_downlink_capacity () =
-  List.fold_left maxi 0 (Fifo.to_list download_history)
+  List.fold_left max 0 (Fifo.to_list download_history)
 
 
 let new_tag name v =
@@ -732,11 +682,6 @@ let partial_chunk c =
   | VerificationBitmap.State_complete | VerificationBitmap.State_verified -> 
       false
 
-module Connections = struct
-
-  end
-
-
 let parse_magnet url =
   let url = Url.of_string url in
   if url.Url.short_file = "magnet:" then
@@ -890,18 +835,69 @@ let _ =
       activity := new_activity ()
   )
 
-module HashMagic = Weak.Make(struct
+module StringIntern = Weak.Make(struct
         type t = string
         let hash s = Hashtbl.hash s
         let equal x y = x = y
       end)
 
-let files_magic = HashMagic.create 100
+let intern_table = StringIntern.create 1000
+let intern s = StringIntern.merge intern_table s
 
-module HashComments = Weak.Make(struct
-        type t = string
-        let hash s = Hashtbl.hash s
-        let equal x y = x = y
-      end)
+let print_command_result o result =
+  if use_html_mods o then
+    html_mods_table_one_row o.conn_buf "serversTable" "servers" [
+      ("", "srh", result); ]
+  else
+    Printf.bprintf o.conn_buf "%s" result
 
-let file_comments = HashComments.create 1000
+let discover_ip force =
+  if !!discover_ip || force then
+    begin
+      if !verbose then lprintf_nl "started IP discovery";
+      let module H = Http_client in
+      let r = { H.basic_request with
+        H.req_url = Url.of_string "http://ip.discoveryvip.com/ip.asp";
+        H.req_proxy = !CommonOptions.http_proxy;
+        H.req_max_retry = 10;
+        H.req_user_agent = get_user_agent () }
+      in
+      H.wget r (fun file ->
+        if !verbose then lprintf_nl "downloaded IP discovery page, parsing...";
+        Unix2.tryopen_read file (fun cin ->
+          try
+            while true do
+              let line = input_line cin in
+              let search_string = "Your Ip address is  " in
+                try
+                  if Str.string_match (Str.regexp ("^" ^ search_string)) line 0 then
+                    begin
+                      set_client_ip =:=
+                        Ip.of_string (String.sub line (String.length search_string)
+                          ((String.length line) - (String.length search_string)));
+                      last_high_id := !!set_client_ip;
+                      if !verbose then lprintf_nl "discovered IP %s" (Ip.to_string !!set_client_ip)
+                    end
+                with e -> lprintf_nl "IP discovery parse error: %s" (Printexc2.to_string e)
+            done
+          with End_of_file -> ())
+      )
+    end
+
+let _ =
+  Heap.add_memstat "CommonGlobals" (fun level buf ->
+      let counter = ref 0 in
+      StringIntern.iter (fun f -> incr counter;) intern_table;
+      Printf.bprintf buf "  intern_table: %d\n" !counter;
+      Printf.bprintf buf " core_gui_fifo: %d\n" (Fifo.length core_gui_fifo);
+      Printf.bprintf buf " gui_core_fifo: %d\n" (Fifo.length gui_core_fifo);
+      Printf.bprintf buf " chat_message_fifo: %d\n" (Fifo.length chat_message_fifo);
+      Printf.bprintf buf " upload_history: %d\n" (Fifo.length upload_history);
+      Printf.bprintf buf " download_history: %d\n" (Fifo.length download_history);
+      Printf.bprintf buf " upload_h_history: %d\n" (Fifo.length upload_h_history);
+      Printf.bprintf buf " download_h_history: %d\n" (Fifo.length download_h_history);
+      Printf.bprintf buf " bandwidth_samples: %d\n" (Fifo.length bandwidth_samples);
+      Printf.bprintf buf " short_delay_bandwidth_samples: %d\n" (Fifo.length short_delay_bandwidth_samples);
+      Printf.bprintf buf " dummy_sample: %d\n" (Array.length dummy_sample);
+      Printf.bprintf buf " activities: %d\n" (Fifo.length activities);
+  )
