@@ -211,7 +211,13 @@ let _ =
   let conv_filename = Charset.to_locale filename in
   if filename <> conv_filename then Charset.conversion_enabled := false;
 
-  Unix2.can_write_to_directory (Filename2.temp_directory ());
+  (try
+     ignore (Sys.getenv "MLDONKEY_TEMP")
+   with Not_found ->
+     Unix.putenv "MLDONKEY_TEMP" ((Filename.basename Sys.argv.(0)) ^ "_tmp")
+  );
+
+  Unix2.can_write_to_directory (Filename2.temp_dir_name ());
 
   if (String2.starts_with (Filename.basename Sys.argv.(0)) "mlnet") then begin
     if Sys.file_exists pid_filename then begin
@@ -603,6 +609,10 @@ let max_hard_download_rate_2 = define_option current_section ["max_hard_download
 let max_opened_connections = define_option current_section ["max_opened_connections"]
   "Maximal number of opened connections"
     int_option 200
+    
+let max_opened_connections_2 = define_option current_section ["max_opened_connections_2"]
+  "Second maximal number of opened connections for easy toggling (use bw_toggle)"
+    int_option 100
 
 let max_indirect_connections = define_option current_section ["max_indirect_connections"]
   "Amount of indirect connections in percent (min 30, max 70) of max_opened_connections"
@@ -613,7 +623,8 @@ let max_upload_slots = define_option current_section ["max_upload_slots"]
     int_option 5
 
 let max_release_slots = define_option current_section ["max_release_slots"]
-  "How many percent of upload slots can be used for downloading files tagged as release"
+  "How many percent of upload slots can be used for downloading files
+  tagged as release, maximum 75%"
     percent_option 20
 
 let friends_upload_slot = define_option current_section ["friends_upload_slot"]
@@ -1178,6 +1189,10 @@ let pause_new_downloads = define_option current_section ["pause_new_downloads"]
   will be set to false on core start."
     bool_option false
 
+let release_new_downloads = define_option current_section ["release_new_downloads"]
+  "Set to true if you want to activate the release slot feature for all new downloads."
+    bool_option false
+
 (*  emulate_sparsefiles does not work, temporarily disabled
 let emulate_sparsefiles = define_expert_option current_section ["emulate_sparsefiles"]
   "Set to true if you want MLdonkey to emulate sparse files on your disk.
@@ -1215,12 +1230,27 @@ let file_completed_cmd = define_option current_section ["file_completed_cmd"]
     $INCOMING  - directory used for commit
     $NETWORK   - network used for downloading
     $ED2K_HASH - ed2k hash if MD4 is known
+    $FILE_OWNER - user who started the download
+    $FILE_GROUP - group the file belongs to
+    $USER_MAIL - mail address of file_owner
   "
     string_option ""
 
 let file_started_cmd = define_option current_section ["file_started_cmd"]
   "The command which is called when a download is started. Arguments
-  are '-file <num>'"
+  are '-file <num>'
+  Also these environment variables can be used (prefered way):
+    $TEMPNAME  - temp file name, including path
+    $FILEID    - same as $1
+    $FILESIZE  - same as $2
+    $FILENAME  - same as $3
+    $FILEHASH  - internal hash
+    $NETWORK   - network used for downloading
+    $ED2K_HASH - ed2k hash if MD4 is known
+    $FILE_OWNER - user who started the download
+    $FILE_GROUP - group the file belongs to
+    $USER_MAIL - mail address of file_owner
+  "
     string_option ""
 
 
@@ -1236,17 +1266,27 @@ let current_section = startup_section
 let run_as_user = define_option current_section ["run_as_user"]
   ~restart: true
   "The login of the user you want mldonkey to run as, after the ports
-  have been bound (can be use not to run with root priviledges when
+  have been bound (can be used not to run with root privileges when
   a port < 1024 is needed)"
     string_option ""
 
 let run_as_useruid = define_option current_section ["run_as_useruid"]
   ~restart: true
   "The UID of the user (0=disabled) you want mldonkey to run as, after the ports
-  have been bound (can be use not to run with root priviledges when
+  have been bound (can be used not to run with root privileges when
   a port < 1024 is needed)"
     int_option 0
 
+let run_as_group = define_option current_section ["run_as_group"]
+  ~restart: true
+  "The group of run_as_user user to be used"
+    string_option ""
+
+let run_as_groupgid = define_option current_section ["run_as_groupgid"]
+  ~restart: true
+  "The group of run_as_user user to be used"
+    int_option 0
+
 let ask_for_gui = define_option current_section ["ask_for_gui"]
   "Ask for GUI start"
     bool_option false
@@ -1728,6 +1768,9 @@ let _ =
     end;
     calc_real_max_indirect_connections ()
   );
+  option_hook max_release_slots (fun _ ->
+    if !!max_release_slots > 75 then max_release_slots =:= 75
+  );
   option_hook min_reask_delay (fun _ ->
     if !!min_reask_delay < 600 then min_reask_delay =:= 600
   );
@@ -1768,9 +1811,6 @@ let _ =
           close_log ()
         end
   );
-  option_hook max_upload_slots (fun _ ->
-      if !!max_upload_slots < 3 then
-        max_upload_slots =:= 3);
   option_hook buffer_writes_threshold (fun _ ->
       Unix32.max_buffered := Int64.of_int (1024 * !!buffer_writes_threshold));
   option_hook log_size (fun _ ->
