@@ -551,6 +551,7 @@ and network = {
     mutable op_network_ports : (unit -> (int * string) list);
     mutable op_network_porttest_start : (unit -> unit);
     mutable op_network_porttest_result : (unit -> network_porttest);
+    mutable op_network_check_upload_slots : (unit -> unit);
   }
 
 and ui_user = {
@@ -818,7 +819,7 @@ and event =
 | File_remove_source_event of file * client
 | Server_new_user_event of server * user
 | Search_new_result_event of gui_result_handler * gui_events * int * result
-
+| Root_console_message_event of string
 | Console_message_event of string
   
 exception FormatFound of format
