@@ -1083,7 +1083,7 @@ let simple_print_file_list finished buf files format =
       ) files)
 
 let print_bw_stats buf =
-  Printf.bprintf buf "Down: %.1f KB/s ( %d + %d ) | Up: %.1f KB/s ( %d + %d ) | Shared: %d/%s | Uploaded: %s"
+  Printf.bprintf buf "Down: %.1f KB/s ( %d + %d ) | Up: %.1f KB/s ( %d + %d ) | Shared: %d/%s | Downloaded: %s | Uploaded: %s"
        (( (float_of_int !udp_download_rate) +. (float_of_int !control_download_rate)) /. 1024.0)
          !udp_download_rate
          !control_download_rate
@@ -1092,6 +1092,7 @@ let print_bw_stats buf =
          !control_upload_rate
          !nshared_files
   (size_of_int64 !nshared_bytes)
+  (size_of_int64 !download_counter)
   (size_of_int64 !upload_counter)
 
 let console_topic () =
@@ -1969,12 +1970,21 @@ let runinfo html buf o =
   let list = ref [] in
   tack list
     (
-      "User:\t\t", 
-      Printf.sprintf "%s (%s) - uptime: %s" 
+      "MLDonkey user:\t\t", 
+      Printf.sprintf "%s (%s) - uptime: %s%s"
 	o.conn_user.ui_user.user_name
 	(if has_empty_password o.conn_user.ui_user then "Warning: empty Password"
 	else "PW Protected")
 	(Date.time_to_string (last_time () - start_time) "verbose")
+        (
+        let user_group =
+          (try Some (Unix.getpwuid (Unix.getuid())).Unix.pw_name with _ -> None),
+          (try Some (Unix.getgrgid (Unix.getgid())).Unix.gr_name with _ -> None)
+        in
+        match user_group with
+        | (Some g, Some u) -> Printf.sprintf " - running as %s:%s" u g
+        | _ -> ""
+        )
     );
   tack list
     (
@@ -2127,6 +2137,7 @@ let diskinfo html buf =
   let list = ref [] in
   ignore (search_incoming_files ());
   ignore (search_incoming_directories ());
+  list := (Filename2.temp_dir_name (), "$MLDONKEY_TEMP") :: !list;
   List.iter (fun dir ->
      list := (dir.shdir_dirname, (Printf.sprintf "shared (%s)" dir.shdir_strategy))
      :: !list) !!shared_directories;
@@ -2511,7 +2522,7 @@ let print_upstats o list server =
 	  ("", "sr ar", Printf.sprintf "%5.1f" ( if size < 1.0 then 0.0 else (uploaded *. 100.) /. size));
 	  ("", "sr", Printf.sprintf "\\<a href=\\\"preview_upload?q=%d\\\"\\>P\\</a\\>" impl.impl_shared_num);
 	  ("", "sr", (if impl.impl_shared_id = Md4.null then
-		        (Filename.basename impl.impl_shared_codedname)
+		        (shorten (Filename.basename impl.impl_shared_codedname) !!max_name_len)
 		      else
 			Printf.sprintf "\\<a href=\\\"%s\\\"\\>%s\\</a\\>"
 			  ed2k (shorten (Filename.basename impl.impl_shared_codedname) !!max_name_len)));
