@@ -149,6 +149,28 @@ let op_file_commit file new_name =
               file.file_torrent_diskname new_torrent_diskname));
       file.file_torrent_diskname <- new_torrent_diskname;
 
+      (* update file_shared with new path to commited file *)
+      match file.file_shared with
+      | None -> ()
+      | Some old_impl ->
+	begin
+	  let impl = {
+	    impl_shared_update = 1;
+	    impl_shared_fullname = file_disk_name file;
+	    impl_shared_codedname = old_impl.impl_shared_codedname;
+	    impl_shared_size = file_size file;
+	    impl_shared_id = Md4.null;
+	    impl_shared_num = 0;
+	    impl_shared_uploaded = old_impl.impl_shared_uploaded;
+	    impl_shared_ops = shared_ops;
+	    impl_shared_val = file;
+	    impl_shared_requests = old_impl.impl_shared_requests;
+	    impl_shared_file = Some (as_file file);
+	    impl_shared_servers = [];
+	  } in
+	  file.file_shared <- Some impl;
+          replace_shared old_impl impl;
+	end
     end 
 
 let op_file_print file o =
@@ -158,7 +180,7 @@ let op_file_print file o =
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
     ("Filename", "sr br", "Filename");
-    ("", "sr", (Charset.safe_convert file.file_encoding file.file_name)) ];
+    ("", "sr", file.file_name) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
@@ -171,25 +193,30 @@ let op_file_print file o =
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   let tracker_header_printed = ref false in
   List.iter (fun tracker ->
-    let tracker_text, tracker_error =
-      (match tracker.tracker_status with
-	  Disabled s | Disabled_mld s | Disabled_failure s -> 
-	    Printf.sprintf "disabled: %s" tracker.tracker_url, s
-	| _ -> tracker.tracker_url, "")
+    let tracker_text =
+      match tracker.tracker_status with
+        | Disabled s | Disabled_mld s ->
+            Printf.sprintf "\\<font color=\\\"red\\\"\\>disabled: %s\\<br\\>\\--error: %s\\</font\\>" tracker.tracker_url s
+        | Disabled_failure (i,s) -> 
+            Printf.sprintf "\\<font color=\\\"red\\\"\\>disabled: %s\\<br\\>\\--error: %s (try %d)\\</font\\>" tracker.tracker_url s i
+        | _ ->
+            Printf.sprintf "enabled: %s" tracker.tracker_url
+
     in
     html_mods_td buf [
       (if not !tracker_header_printed then
-        ("Tracker(s) (mouseover for errors)", "sr br", "Tracker(s)")
+        ("Tracker(s)", "sr br", "Tracker(s)")
        else
-	("", "sr br", ""));
-      (tracker_error, "sr", tracker_text)];
+        ("", "sr br", "")
+      );
+      (tracker.tracker_url, "sr", tracker_text)];
     Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
     tracker_header_printed := true;
   ) file.file_trackers;
 
   html_mods_td buf [
     ("Torrent Filename", "sr br", "Torrent Fname");
-    ("", "sr", (Charset.safe_convert file.file_encoding file.file_torrent_diskname)) ];
+    ("", "sr", file.file_torrent_diskname) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
 
@@ -197,14 +224,14 @@ let op_file_print file o =
     ("Comment", "sr br", "Comment");
     ("", "sr", match file.file_comment with
         "" -> "-"
-      | _ -> (Charset.safe_convert file.file_encoding file.file_comment)) ];
+      | _ -> file.file_comment) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
     ("Created by", "sr br", "Created by");
     ("", "sr", match file.file_created_by with
         "" -> "-"
-      | _ -> (Charset.safe_convert file.file_encoding file.file_created_by)) ];
+      | _ -> file.file_created_by) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
@@ -216,7 +243,7 @@ let op_file_print file o =
     ("Modified by", "sr br", "Modified by");
     ("", "sr", match file.file_modified_by with
         "" -> "-"
-      | _ -> (Charset.safe_convert file.file_encoding file.file_modified_by)) ];
+      | _ -> file.file_modified_by) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
@@ -225,6 +252,11 @@ let op_file_print file o =
         "" -> "-"
       | _ -> file.file_encoding) ];
 
+  Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+  html_mods_td buf [
+    ("Piece size", "sr br", "Piece size");
+    ("", "sr", Int64.to_string file.file_piece_size) ];
+
   let rec print_first_tracker l =
     match l with
       | [] -> ()
@@ -339,20 +371,19 @@ let op_file_print file o =
   Printf.bprintf buf "Trackers:\n";
   List.iter (fun tracker ->
     match tracker.tracker_status with
-      Disabled s | Disabled_mld s | Disabled_failure s -> 
-	Printf.bprintf buf "%s, disabled: %s\n" tracker.tracker_url s
-      | _ -> Printf.bprintf buf "%s\n" tracker.tracker_url
+    | Disabled s | Disabled_mld s ->
+        Printf.bprintf buf "%s, disabled: %s\n" tracker.tracker_url s
+    | Disabled_failure (i,s) -> 
+        Printf.bprintf buf "%s, disabled (try %d): %s\n" tracker.tracker_url i s
+    | _ -> Printf.bprintf buf "%s\n" tracker.tracker_url
   ) file.file_trackers;
-  let s = Charset.safe_convert file.file_encoding file.file_torrent_diskname in
-  if s <> "" then Printf.bprintf buf "Torrent diskname: %s\n" s;
-  let s = Charset.safe_convert file.file_encoding file.file_comment in
-  if s <> "" then Printf.bprintf buf "Comment: %s\n" s;
-  let s = Charset.safe_convert file.file_encoding file.file_created_by in
-  if s <> "" then Printf.bprintf buf "Created by %s\n" s;
+  if file.file_torrent_diskname <> "" then
+    Printf.bprintf buf "Torrent diskname: %s\n" file.file_torrent_diskname;
+  if file.file_comment <> "" then Printf.bprintf buf "Comment: %s\n" file.file_comment;
+  if file.file_created_by <> "" then Printf.bprintf buf "Created by %s\n" file.file_created_by;
   let s = Date.to_string (Int64.to_float file.file_creation_date) in
   if s <> "" then Printf.bprintf buf "Creation date: %s\n" s;
-  let s = Charset.safe_convert file.file_encoding file.file_modified_by in
-  if s <> "" then Printf.bprintf buf "Modified by %s\n" s;
+  if file.file_modified_by <> "" then Printf.bprintf buf "Modified by %s\n" file.file_modified_by;
   if file.file_encoding <> "" then Printf.bprintf buf "Encoding: %s\n" file.file_encoding;
   if file.file_files <> [] then Printf.bprintf buf "Subfiles: %d\n" (List.length file.file_files);
   let cntr = ref 0 in
@@ -405,9 +436,11 @@ let op_file_print_sources file o =
         ( "0", "srh br", "Client software", "Soft" ) ;
         ( "0", "srh", "IP address", "IP address" ) ;
         ( "0", "srh br ar", "Port", "Port" ) ;
-        ] @ (if !Geoip.active then [( "0", "srh br ar", "Country Code/Name", "CC" )] else []) @ [
-        ( "1", "srh ar", "Total UL bytes to this client for all files", "UL" ) ;
-        ( "1", "srh ar br", "Total DL bytes from this client for all files", "DL" ) ;
+        ] @ (if Geoip.active () then [( "0", "srh br ar", "Country Code/Name", "CC" )] else []) @ [
+        ( "1", "srh ar", "Total UL bytes to this client for all files", "tUL" ) ;
+        ( "1", "srh ar br", "Total DL bytes from this client for all files", "tDL" ) ;
+        ( "1", "srh ar", "Session UL bytes to this client for all files", "sUL" ) ;
+        ( "1", "srh ar br", "Session DL bytes from this client for all files", "sDL" ) ;
         ( "0", "srh ar", "Interested [T]rue, [F]alse", "I" ) ;
         ( "0", "srh ar", "Choked [T]rue, [F]alse", "C" ) ;
         ( "1", "srh br ar", "Allowed to write", "A" ) ;
@@ -446,20 +479,25 @@ let op_file_print_sources file o =
       html_mods_table_header buf "sourcesTable" "sources al" header_list;
 
       Hashtbl.iter (fun _ c ->
+          let cinfo = client_info (as_client c) in
 	  if use_html_mods o then
           Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr());
 
           let btos b = if b then "T" else "F" in
-          let cc,cn = Geoip.get_country (fst c.client_host) in
+          let cc,cn = Geoip.get_country_code_name cinfo.GuiTypes.client_country_code in
           let td_list = [
             ("", "sr br ar", Printf.sprintf "%d" (client_num c));
             ("", "sr br", (Sha1.to_string c.client_uid));
             ("", "sr br", Printf.sprintf "%s %s" (brand_to_string c.client_brand) c.client_release);
             ("", "sr", (Ip.to_string (fst c.client_host)));
             ("", "sr br ar", Printf.sprintf "%d" (snd c.client_host));
-            ] @ (if !Geoip.active then [( cn, "sr br", cc)] else []) @ [
-            ("", "sr ar", (size_of_int64 c.client_uploaded));
-            ("", "sr ar br", (size_of_int64 c.client_downloaded));
+            ] @ (if Geoip.active () then 
+              [( cn, "sr br", if use_html_mods o then CommonPictures.flag_html cc else cc)]
+                 else []) @ [
+            ("", "sr ar", (size_of_int64 c.client_total_uploaded));
+            ("", "sr ar br", (size_of_int64 c.client_total_downloaded));
+            ("", "sr ar", (size_of_int64 c.client_session_uploaded));
+            ("", "sr ar br", (size_of_int64 c.client_session_downloaded));
             ("", "sr", (btos c.client_interested));
             ("", "sr", (btos c.client_choked));
             ("", "sr br ar", (Int64.to_string c.client_allowed_to_write));
@@ -546,6 +584,9 @@ let op_file_info file =
     P.file_chunks = (match file.file_swarmer with
     | None -> None 
     | Some swarmer -> Some (CommonSwarming.chunks_verified_bitmap swarmer));
+    P.file_chunk_size = (match file.file_swarmer with
+    | None -> None 
+    | Some t -> Some (List.map (fun t -> t.CommonSwarming.t_chunk_size) t.CommonSwarming.t_s.CommonSwarming.s_networks));
     P.file_availability =
     [network.network_num,(match file.file_swarmer with
           None -> "" | Some swarmer ->
@@ -580,6 +621,7 @@ let op_ft_info ft =
     P.file_sources = None;
     P.file_download_rate = 0.;
     P.file_chunks = None;
+    P.file_chunk_size = None;
     P.file_availability =  [network.network_num, ""];
     P.file_format = FormatNotComputed 0;
     P.file_chunks_age = [| last_time () |];
@@ -597,7 +639,8 @@ let op_ft_info ft =
 
 
 
-let load_torrent_string s user =
+let load_torrent_string s user group =
+  if !verbose then lprintf_nl "load_torrent_string";
   let file_id, torrent = BTTorrent.decode_torrent s in
 
   (* Save the torrent, because we later want to put
@@ -607,28 +650,34 @@ let load_torrent_string s user =
     String2.check_prefix url "http://" in
   List.iter (fun url -> if can_handle_tracker url then torrent_is_usable := true)
     (if torrent.torrent_announce_list <> [] then torrent.torrent_announce_list else [torrent.torrent_announce]);
-  if not !torrent_is_usable then raise Torrent_can_not_be_used;
+  if not !torrent_is_usable then raise (Torrent_can_not_be_used torrent.torrent_name);
 
-  let torrent_diskname = Filename.concat
-    downloads_directory
-    torrent.torrent_name ^ ".torrent"
+  let torrent_diskname =
+    let fs = Unix32.filesystem downloads_directory in
+    let namemax =
+      match Unix32.fnamelen downloads_directory with
+      | None -> 0
+      | Some v -> v
+    in
+    Filename.concat downloads_directory
+    (Filename2.filesystem_compliant torrent.torrent_name fs namemax) ^ ".torrent"
     in
   if Sys.file_exists torrent_diskname then
     begin
       if !verbose then lprintf_nl "load_torrent_string: %s already exists, ignoring" torrent_diskname;
-      raise Torrent_already_exists
+      raise (Torrent_already_exists torrent.torrent_name)
     end;
   File.from_string torrent_diskname s;
 
   if !verbose then
     lprintf_nl "Starting torrent download with diskname: %s"
         torrent_diskname;
-  let file = new_download file_id torrent torrent_diskname user in
+  let file = new_download file_id torrent torrent_diskname user group in
   BTClients.get_sources_from_tracker file;
   CommonInteractive.start_download (file_find (file_num file));
   file
 
-let load_torrent_file filename user =
+let load_torrent_file filename user group =
   if !verbose then
     lprintf_nl "load_torrent_file %s" filename;
   let s = File.to_string filename in
@@ -637,7 +686,7 @@ let load_torrent_file filename user =
   if Sys.file_exists filename
       && (Filename.dirname filename) = downloads_directory then
     Sys.remove filename;
-  ignore (load_torrent_string s user)
+  ignore (load_torrent_string s user group)
 
 let parse_tracker_reply file t filename =
 (*This is the function which will be called by the http client
@@ -699,11 +748,12 @@ let try_share_file torrent_diskname =
             else
               iter tail
       in
-      iter !!shared_directories
+      iter (shared_directories_including_user_commit ())
     in
 
+    let user = CommonUserDb.admin_user () in
     let file = new_file file_id torrent torrent_diskname
-        filename FileShared CommonUserDb.admin_user in
+        filename FileShared user user.user_default_group in
     if !verbose_share then lprintf_file_nl (as_file file) "Sharing file %s" filename;
     BTClients.connect_trackers file "started"
       (parse_tracker_reply file)
@@ -720,11 +770,6 @@ let try_share_file torrent_diskname =
         with _ ->
           (lprintf_nl "Failed to rename %s to %s"
               torrent_diskname new_torrent_diskname));
-(*
-      (try Sys.remove torrent_diskname with e ->
-          if !verbose_share then
-            lprintf_nl "Error: %s while removing torrent %s" (Printexc2.to_string e) s)
-*)
   | e ->
       lprintf_nl "Cannot share torrent %s for %s"
         torrent_diskname (Printexc2.to_string e)
@@ -733,13 +778,9 @@ let try_share_file torrent_diskname =
   automatically contact the tracker. *)
 let share_files _ =
   if !verbose_share then lprintf_nl "share_files";
-  List.iter (fun dir ->
-      let filenames = Unix2.list_directory dir in
-      List.iter (fun file ->
-          let filename = Filename.concat dir file in
-          try_share_file filename
-      ) filenames
-  ) [seeded_directory];
+  List.iter (fun file ->
+    try_share_file (Filename.concat seeded_directory file)
+  ) (Unix2.list_directory seeded_directory);
   let shared_files_copy = !current_files in
  (* if the torrent is gone while the file is still shared, remove the share *)
   List.iter (fun file ->
@@ -762,11 +803,16 @@ let scan_new_torrents_directory () =
     let file_basename = Filename.basename file in
     if not (Unix2.is_directory file) then
     try
-      let user = fst (Unix32.owner file) in
-      load_torrent_file file (try CommonUserDb.user2_user_find user with Not_found -> CommonUserDb.admin_user);
+      let file_owner = fst (Unix32.owner file) in
+      let user =
+        try
+          CommonUserDb.user2_user_find file_owner
+        with Not_found -> CommonUserDb.admin_user ()
+      in
+      load_torrent_file file user user.user_default_group;
       (try Sys.remove file with _ -> ())
     with 
-      Torrent_can_not_be_used ->
+      Torrent_can_not_be_used _ ->
 	Unix2.rename file (Filename.concat old_directory file_basename);
 	lprintf_nl "Torrent %s does not have valid tracker URLs, moved to torrents/old ..." file_basename
     | e ->
@@ -781,12 +827,12 @@ let retry_all_ft () =
           lprintf_nl "ft_retry: exception %s" (Printexc2.to_string e)
   ) ft_by_num
 
-let load_torrent_from_web r user ft =
+let load_torrent_from_web r user group ft =
   let module H = Http_client in
   H.wget r (fun filename ->
       if ft_state ft = FileDownloading then begin
-          load_torrent_file filename user;
-          file_cancel (as_ft ft) CommonUserDb.admin_user
+          load_torrent_file filename user group;
+          file_cancel (as_ft ft) (CommonUserDb.admin_user ())
         end)
 
 let valid_torrent_extension url =
@@ -799,7 +845,7 @@ let get_regexp_string text r =
   let b = Str.group_end 1 in
   String.sub text a (b - a)
 
-let op_network_parse_url url user =
+let op_network_parse_url url user group =
   let location_regexp = "Location: \\(.*\\)" in
   try
     let real_url = get_regexp_string url (Str.regexp location_regexp) in
@@ -835,8 +881,8 @@ let op_network_parse_url url user =
 
         let file_diskname = Filename.basename u.Url.short_file in
         let ft = new_ft file_diskname user in
-        ft.ft_retry <- (load_torrent_from_web r user);
-        load_torrent_from_web r user ft;
+        ft.ft_retry <- (load_torrent_from_web r user group);
+        load_torrent_from_web r user group ft;
         "started download", true
       )
     else
@@ -847,11 +893,11 @@ let op_network_parse_url url user =
         try
           if !verbose then lprintf_nl "Not_found and trying to load %s" url;
           try
-            load_torrent_file url user;
+            load_torrent_file url user group;
             "", true
           with
-	    Torrent_already_exists -> "A torrent with this name is already in the download queue", false
-	  | Torrent_can_not_be_used -> "This torrent does not have valid tracker URLs", false
+	    Torrent_already_exists _ -> "A torrent with this name is already in the download queue", false
+	  | Torrent_can_not_be_used _ -> "This torrent does not have valid tracker URLs", false
         with e ->
           lprintf_nl "Exception %s while 2nd loading" (Printexc2.to_string e);
 	  let s = Printf.sprintf "Can not load load torrent file: %s"
@@ -869,19 +915,23 @@ let op_network_parse_url url user =
        s, false
 
 let op_client_info c =
+  check_client_country_code c;
   let module P = GuiTypes in
   let (ip,port) = c.client_host in
   { (impl_client_info c.client_client) with
 
     P.client_network = network.network_num;
     P.client_kind = Known_location (ip,port);
+    P.client_country_code = c.client_country_code;
     P.client_state = client_state (as_client c);
     P.client_type = client_type c;
     P.client_name = (Printf.sprintf "%s:%d" (Ip.to_string ip) port);
     P.client_software = (brand_to_string c.client_brand);
     P.client_release = c.client_release;
-    P.client_downloaded = c.client_downloaded;
-    P.client_uploaded = c.client_uploaded;
+    P.client_total_downloaded = c.client_total_downloaded;
+    P.client_total_uploaded = c.client_total_uploaded;
+    P.client_session_downloaded = c.client_session_downloaded;
+    P.client_session_uploaded = c.client_session_uploaded;
     P.client_upload = Some (c.client_file.file_name);
     P.client_connect_time = c.client_connect_time;
 
@@ -906,11 +956,11 @@ let op_client_dprint c o file =
   let cc = as_client c in
   client_print cc o;
   Printf.bprintf buf (_b "\n%18sDown  : %-10s                  Uploaded: %-10s  Ratio: %s%1.1f (%s)\n") ""
-    (Int64.to_string c.client_downloaded)
-  (Int64.to_string c.client_uploaded)
-  (if c.client_downloaded > c.client_uploaded then "-" else "+")
-  (if c.client_uploaded > Int64.zero then 
-     Int64.to_float (c.client_downloaded // c.client_uploaded) 
+    (Int64.to_string c.client_total_downloaded)
+  (Int64.to_string c.client_total_uploaded)
+  (if c.client_total_downloaded > c.client_total_uploaded then "-" else "+")
+  (if c.client_total_uploaded > Int64.zero then 
+     Int64.to_float (c.client_total_downloaded // c.client_total_uploaded) 
    else 1.)
   ("BT");
   (Printf.bprintf buf (_b "%18sFile  : %s\n") "" info.GuiTypes.file_name)
@@ -929,7 +979,7 @@ let op_client_dprint_html c o file str =
         show_emulemods_column := true
     end;
 
-    let cc,cn = Geoip.get_country (fst c.client_host) in
+    let cc,cn = Geoip.get_country_code_name cinfo.GuiTypes.client_country_code in
 
     html_mods_td buf ([
         ("", "srb ar", Printf.sprintf "%d" (client_num c));
@@ -947,9 +997,11 @@ let op_client_dprint_html c o file str =
         ("", "sr", "D");
         ("", "sr", "N");
         ("", "sr", (Ip.to_string (fst c.client_host)));
-        ] @ (if !Geoip.active then [(cn, "sr", cc)] else []) @ [
-        ("", "sr ar", (size_of_int64 c.client_uploaded));
-        ("", "sr ar", (size_of_int64 c.client_downloaded));
+        ] @ (if Geoip.active () then [(cn, "sr", CommonPictures.flag_html cc)] else []) @ [
+        ("", "sr ar", (size_of_int64 c.client_total_uploaded));
+        ("", "sr ar", (size_of_int64 c.client_total_downloaded));
+        ("", "sr ar", (size_of_int64 c.client_session_uploaded));
+        ("", "sr ar", (size_of_int64 c.client_session_downloaded));
         ("", "sr", info.GuiTypes.file_name); ]);
     true
 
@@ -1039,6 +1091,35 @@ let commands =
         _s ""
     ), _s ":\t\t\t\tprint all .torrent files on this server";
 
+    "print_torrent", "Network/Bittorrent", Arg_one (fun arg o ->
+      if CommonUserDb.user2_is_admin o.conn_user.ui_user then begin
+      let file =
+        try
+          Some (as_file_impl (file_find (int_of_string arg)))
+        with _ -> None
+      in
+      match file with
+      | None -> Printf.sprintf "file %s not found" arg
+      | Some file ->
+        (
+        if use_html_mods o then begin
+          html_mods_cntr_init ();
+          html_mods_table_header o.conn_buf "sourcesInfo" "sourcesInfo" [
+            ( "0", "srh br", "File Info", "Info" ) ;
+            ( "0", "srh", "Value", "Value" ) ]
+        end;
+        op_file_print file.impl_file_val o;
+        if use_html_mods o then begin
+          Printf.bprintf o.conn_buf "\\</tr\\>\\</table\\>\\</div\\>";
+          Printf.bprintf o.conn_buf "\\</td\\>\\</tr\\>\\</table\\>\\</div\\>\\<br\\>"
+        end
+        );
+      ""
+      end else
+      begin print_command_result o "You are not allowed to use print_torrent";
+      "" end
+    ), _s "<num> :\t\t\tshow internal data of .torrent file";
+
     "seeded_torrents", "Network/Bittorrent", Arg_none (fun o ->
       if CommonUserDb.user2_is_admin o.conn_user.ui_user then begin
       List.iter (fun file ->
@@ -1047,7 +1128,7 @@ let commands =
       ) !current_files;
       _s "done"
       end else
-      begin print_command_result o o.conn_buf "You are not allowed to use seeded_torrents";
+      begin print_command_result o "You are not allowed to use seeded_torrents";
       "" end
     ), _s ":\t\t\tprint all seeded .torrent files on this server";
 
@@ -1084,13 +1165,13 @@ let commands =
       let buf = o.conn_buf in
       if Sys.file_exists url then
         begin
-	  load_torrent_file url o.conn_user.ui_user;
+	  load_torrent_file url o.conn_user.ui_user o.conn_user.ui_user.user_default_group;
           Printf.bprintf buf "loaded file %s\n" url
 	end
       else
         begin
           let url = "Location: " ^ url ^ "\nContent-Type: application/x-bittorrent" in
-	  let result = fst (op_network_parse_url url o.conn_user.ui_user) in
+	  let result = fst (op_network_parse_url url o.conn_user.ui_user o.conn_user.ui_user.user_default_group) in
           Printf.bprintf buf "%s\n" result
 	end;
       _s ""
@@ -1171,10 +1252,11 @@ let op_gui_message s user =
       let text = String.sub s 2 (String.length s - 2) in
       if !verbose then lprintf_nl "received torrent from gui...";
       (try
-        ignore (load_torrent_string text user)
+        let file = load_torrent_string text user user.user_default_group in
+        raise (Torrent_started file.file_name)
       with e -> (match e with
-	  Torrent_can_not_be_used -> lprintf_nl "Loading torrent from GUI: this torrent can not be used"
-	| Torrent_already_exists -> lprintf_nl "Loading torrent from GUI: this torrent is already in download queue"
+	| Torrent_can_not_be_used s -> lprintf_nl "Loading torrent from GUI: torrent %s can not be used" s
+	| Torrent_already_exists s -> lprintf_nl "Loading torrent from GUI: torrent %s is already in download queue" s
 	| _ -> ());
 	raise e)
   | 1 -> (* 34+ *)
@@ -1191,6 +1273,8 @@ let _ =
   ft_ops.op_file_cancel <- op_ft_cancel;
   ft_ops.op_file_commit <- op_ft_commit;
   ft_ops.op_file_info <- op_ft_info;
+  ft_ops.op_file_active_sources <- (fun _ -> []);
+  ft_ops.op_file_all_sources <- (fun _ -> []);
 
   file_ops.op_file_all_sources <- op_file_all_sources;
   file_ops.op_file_files <- op_file_files;
@@ -1208,6 +1292,16 @@ let _ =
         None -> None
       | Some sh -> Some (as_shared sh)
   );
+  file_ops.op_file_download_order <- (fun file strategy ->
+      match file.file_swarmer with
+      | None -> None
+      | Some s ->
+          (match strategy with
+          (* return current strategy *)
+          | None -> Some (CommonSwarming.get_strategy s)
+          | Some strategy -> CommonSwarming.set_strategy s strategy;
+                      Some (CommonSwarming.get_strategy s))
+  );
 
   network.op_network_gui_message <- op_gui_message;
   network.op_network_connected <- op_network_connected;
@@ -1217,7 +1311,7 @@ let _ =
   network.op_network_forget_search <- (fun s -> ());
   network.op_network_connect_servers <- (fun s -> ());
   network.op_network_search <- (fun ss buf -> ());
-  network.op_network_download <- (fun r user -> dummy_file);
+  network.op_network_download <- (fun r user group -> dummy_file);
   network.op_network_recover_temp <- (fun s -> ());
   let clean_exit_started = ref false in
   network.op_network_clean_exit <- (fun s ->
@@ -1254,15 +1348,25 @@ let _ =
 	porttest_result := PorttestResult (last_time (), result)
       )
   );
+  network.op_network_check_upload_slots <- (fun _ -> check_bt_uploaders ());
   client_ops.op_client_info <- op_client_info;
   client_ops.op_client_connect <- op_client_connect;
   client_ops.op_client_disconnect <- op_client_disconnect;
   client_ops.op_client_bprint <- op_client_bprint;
   client_ops.op_client_dprint <- op_client_dprint;
   client_ops.op_client_dprint_html <- op_client_dprint_html;
+  client_ops.op_client_browse <- (fun _ _ -> ());
+  client_ops.op_client_files <- (fun _ -> []);
+  client_ops.op_client_clear_files <- (fun _ -> ());
 
   CommonNetwork.register_commands commands;
 
+  shared_ops.op_shared_state <- (fun file o ->
+    if o.conn_output = HTML then
+      Printf.sprintf "\\<a href=\\\"submit?q=print_torrent+%d\\\"\\>Details\\</a\\>"
+        (file_num file)
+    else Printf.sprintf "Shared using %s" file.file_torrent_diskname
+  );
   shared_ops.op_shared_unshare <- (fun file ->
       (if !verbose_share then lprintf_file_nl (as_file file) "unshare file");
       BTGlobals.unshare_file file);
