@@ -56,7 +56,8 @@ plugin.
 **************************************************************)
 
 let network = CommonNetwork.new_network "ED2K" "Donkey"
-         [
+  ~comment:(if Autoconf.donkey_sui_works () then "SUI" else "noSUI")
+  [
     NetworkHasServers;
     NetworkHasSearch;
     NetworkHasUpload;
@@ -103,6 +104,7 @@ let as_client c = as_client c.client_client
 let as_file file = as_file file.file_file
 let file_priority file = file.file_file.impl_file_priority
 let file_size file = file.file_file.impl_file_size
+let file_is_largefile f = file_size f > old_max_emule_file_size
 let file_downloaded file = file_downloaded (as_file file)
 let file_age file = file.file_file.impl_file_age
 let file_fd file = file_fd (as_file file)
@@ -125,13 +127,12 @@ let tag_client = 200
 let tag_server = 201
 let tag_file   = 202
 
-let page_size = 4096L
-
 let donkey_download_counter = ref Int64.zero
 let donkey_upload_counter = ref Int64.zero
 
 let client_to_client_tags = ref ([] : tag list)
 let client_to_server_tags = ref ([] : tag list)
+let client_to_server_reply_tags = ref ([] : tag list)
 let emule_info =
   let module E = DonkeyProtoClient.EmuleClientInfo in
   {
@@ -140,6 +141,8 @@ let emule_info =
     E.tags = [];
   }
 
+let sec_ident_enabled () = !!enable_sui && (Autoconf.donkey_sui_works ())
+
 let overnet_connectreply_tags = ref ([] :  tag list)
 let overnet_connect_tags = ref ([] :  tag list)
 
@@ -184,8 +187,7 @@ let max_file_groups = 1000
 let master_server = ref (None: DonkeyTypes.server option)
 let udp_sock = ref (None: UdpSocket.t option)
 let listen_sock = ref (None : TcpServerSocket.t option)
-let reversed_sock = ref (None : TcpServerSocket.t option)
-let new_shared = ref false
+let porttest_sock = ref (None : TcpBufferedSocket.t option)
 
 (*************************************************************************)
 (*                                                                       *)
@@ -204,6 +206,8 @@ let clients_by_kind = H.create 127
 let clients_root = ref []
 let servers_by_key = Hashtbl.create 127
 let servers_list = ref ([] : server list)
+let walker_list = ref ([] : server list)
+let delayed_list = ref ([] : server list)
 
 (* let remaining_time_for_clients = ref (60 * 15) *)
 
@@ -230,7 +234,6 @@ in the shared_files_info *)
 let shared_files_info = (Hashtbl.create 127
     : (string * int64 * float, shared_file_info) Hashtbl.t)
 let shared_files = ref ([] : file_to_share list)
-let new_shared_files = ref []
 
 let udp_servers_replies = (Hashtbl.create 127 : (Md4.t, server) Hashtbl.t)
 
@@ -285,11 +288,24 @@ let remove_connected_server c =
 
 let connected_servers () = !connected_server_list
 
+let logged_in_servers () =
+List.filter (fun s -> 
+  match server_state s with
+  | Connected _ -> true
+  | _ -> false) !connected_server_list
+
 let get_udp_sock () =
   match !udp_sock with
     None -> failwith "No UDP socket"
   | Some sock -> sock
 
+let md4_of_array md4s =
+  let s = String.create ((Array.length md4s) * 16) in
+  Array.iteri (fun i v -> 
+    String.blit (Md4.direct_to_string v) 0 s (i*16) 16
+  ) md4s;
+  Md4.string s
+
 (* compute the name used to save the file *)
 
 let update_best_name file =
@@ -311,8 +327,7 @@ let update_best_name file =
           set_file_best_name file best_name "" 0
     with Not_found -> ()
 
-let new_file file_diskname file_state md4 file_size filename writable =
-
+let new_file file_diskname file_state md4 file_size filename writable user group =
   try
       let file = find_file md4 in
       if file.file_diskname <> file_diskname then
@@ -360,7 +375,10 @@ let new_file file_diskname file_state md4 file_size filename writable =
           Unix32.create_sparsefile file_diskname writable
         else
 *)
-          Unix32.create_diskfile file_diskname writable
+          try
+            Unix32.create_diskfile file_diskname writable
+          with e ->
+            failwith (Printf.sprintf "Error: %s" (Printexc2.to_string e))
       in
       let file_size =
         if file_size = Int64.zero then
@@ -379,8 +397,8 @@ let new_file file_diskname file_state md4 file_size filename writable =
       (try
         Unix32.remove t
        with e ->
-    lprintf_nl "Unix32.remove %s exception %s"
-      (file_diskname) (Printexc2.to_string e));
+	 lprintf_nl "Unix32.remove %s exception %s"
+	   (file_diskname) (Printexc2.to_string e));
       Unix32.destroy t;
       failwith (Printf.sprintf "file size %s is too big, exception: %s"
         (size_of_int64 file_size) (Printexc2.to_string e))
@@ -398,10 +416,13 @@ let new_file file_diskname file_state md4 file_size filename writable =
           file_computed_md4s = Array.of_list md4s;
           file_format = FormatNotComputed 0;
           file_sources = DonkeySources.create_file_sources_manager
-            (Md4.to_string md4)
+            (Md4.to_string md4);
+          file_comments = [];
         }
       and file_impl = {
           dummy_file_impl with
+          impl_file_owner = user;
+          impl_file_group = group;
           impl_file_val = file;
           impl_file_ops = file_ops;
           impl_file_age = last_time ();
@@ -409,7 +430,7 @@ let new_file file_diskname file_state md4 file_size filename writable =
           impl_file_fd = Some t;
           impl_file_best_name = Filename.basename file_diskname;
           impl_file_filenames = (if filename = "" then [] else [filename]);
-          impl_file_last_seen = last_time () - 100 * 24 * 3600;
+          impl_file_last_seen = last_time () - 100 * Date.day_in_secs;
         }
       in
 
@@ -427,10 +448,7 @@ let new_file file_diskname file_state md4 file_size filename writable =
                 Verification (Array.of_list (List.map (fun md4 -> Ed2k md4) md4s))
             );
             CommonSwarming.set_verified swarmer (fun nblocks num ->
-                if nblocks = 1 then begin
-                    new_shared_files := file :: !new_shared_files;
-                    file_must_update file
-                  end)
+                if nblocks = 1 then file_must_update file)
       );
 
       update_best_name file;
@@ -463,18 +481,27 @@ let low_id ip =
     | _, _, _, 0 -> true
     | _ -> false
 
-let is_black_address ip port =
+let is_black_address ip port cc =
   !!black_list && not (low_id ip) && (
 (* lprintf "is black ="; *)
-    not (Ip.reachable ip) || (Ip.matches ip !!server_black_list) ||
+    not (Ip.reachable ip) || 
+    (match Ip_set.match_ip !server_black_list_set ip with
+     | Some br -> true 
+     | None -> false) ||
     (List.mem port !!port_black_list) ||
-    (match !Ip.banned ip with
+    (match !Ip.banned (ip, cc) with
         None -> false
       | Some reason ->
           if !verbose_connect then
             lprintf_nl "%s:%d blocked: %s" (Ip.to_string ip) port reason;
           true))
 
+let check_server_country_code s =
+  if Geoip.active () then
+    match s.server_country_code with
+    | None -> s.server_country_code <- Geoip.get_country_code_option s.server_ip
+    | _ -> ()
+
 let new_server ip port =
   let key = (ip) in
   try
@@ -491,8 +518,8 @@ let new_server ip port =
         server_cid = None (* client_ip None *);
         server_port = port;
         server_realport = None;
+        server_country_code = None;
         server_sock = NoConnection;
-        server_nqueries = 0;
         server_search_queries = Fifo.create ();
         server_users_queries = Fifo.create ();
         server_connection_control = new_connection_control ();
@@ -506,20 +533,37 @@ let new_server ip port =
         server_users = [];
         server_master = false;
         server_preferred = false;
-        server_last_message = 0;
         server_queries_credit = 0;
         server_waiting_queries = [];
         server_sent_all_queries = false;
         server_id_requests = Fifo.create ();
         server_flags = 0;
         server_has_zlib = false;
+        server_has_newtags = false;
+        server_has_unicode = false;
+        server_has_related_search = false;
+        server_has_tag_integer = false;
+        server_has_largefiles = false;
         server_version = "";
         server_lowid_users = None;
         server_soft_limit = None;
         server_hard_limit = None;
+        server_obfuscation_tcp = None;
+        server_obfuscation_udp = None;
+        server_sent_shared = [];
         server_max_users = None;
         server_last_ping = 0.;
+        server_next_ping = 0.;
+        server_descping_counter = 0;
         server_ping = 0;
+        server_failed_count = 0;
+        server_udp_ping_challenge = None;
+        server_udp_desc_challenge = None;
+        server_has_get_sources = false;
+        server_has_get_files = false;
+        server_has_get_sources2 = false;
+        server_dynip = "";
+        server_auxportslist = "";
 
       }
       and server_impl =
@@ -532,6 +576,7 @@ let new_server ip port =
       server_add server_impl;
       Heap.set_tag s tag_server;
       Hashtbl.add servers_by_key key s;
+      check_server_country_code s;
       server_must_update s;
       s
 
@@ -545,6 +590,8 @@ let remove_server ip port =
   try
     Hashtbl.remove servers_by_key key;
     servers_list := List2.removeq s !servers_list ;
+    walker_list := List2.removeq s !walker_list;
+    delayed_list := List2.removeq s !delayed_list;
     (match s.server_sock with
         NoConnection -> ()
       | ConnectionWaiting token -> cancel_token token
@@ -552,6 +599,18 @@ let remove_server ip port =
           TcpBufferedSocket.shutdown sock Closed_by_user)
   with _ -> ()
 
+let check_client_country_code c =
+  if Geoip.active () then
+    match c.client_country_code with
+    | None ->
+        (match c.client_kind with
+        | Direct_address (ip,port) ->
+            c.client_country_code <- Geoip.get_country_code_option ip
+        | Indirect_address (_,_,_,_,real_ip) ->
+            c.client_country_code <- Geoip.get_country_code_option real_ip
+        | _ -> ())
+    | _ -> ()
+
 let dummy_client =
   let module D = DonkeyProtoClient in
   let rec c = {
@@ -560,28 +619,28 @@ let dummy_client =
       client_kind = Direct_address (Ip.null, 0);
       client_source = DonkeySources.dummy_source;
       client_ip = Ip.null;
+      client_country_code = None;
       client_md4 = Md4.null;
       client_download = None;
       client_file_queue = [];
       client_tags = [];
       client_name = "";
-      client_last_asked_file = dummy_file;
       client_all_files = None;
       client_rating = 0;
       client_brand = Brand_unknown;
       client_brand_mod = Brand_mod_unknown;
+      client_osinfo = None;
       client_checked = false;
       client_connected = false;
-      client_downloaded = Int64.zero;
-      client_uploaded = Int64.zero;
+      client_session_downloaded = Int64.zero;
+      client_session_uploaded = Int64.zero;
+      client_total_downloaded = Int64.zero;
+      client_total_uploaded = Int64.zero;
       client_banned = false;
-      client_score = 0;
-      client_next_queue = 0;
       client_rank = 0;
       client_connect_time = 0;
       client_requests_sent = 0;
       client_requests_received = 0;
-      client_indirect_address = None;
       client_slot = SlotNotAsked;
       client_debug = false;
       client_pending_messages = [];
@@ -592,6 +651,8 @@ let dummy_client =
       client_sent_challenge = Int64.zero;
       client_public_key = None;
       client_sui_verified = None;
+      client_last_file_req_md4 = None;
+      client_osinfo_sent = false;
       } and
     client_impl = {
       dummy_client_impl with
@@ -602,37 +663,39 @@ let dummy_client =
   in
   c
 
-let create_client key =
+let create_client key cc =
   let module D = DonkeyProtoClient in
-  let s = DonkeySources.find_source_by_uid key in
+  let s = DonkeySources.create_source_by_uid (match key with
+      Indirect_address (server_ip, server_port, id, port, real_ip) -> Indirect_address (server_ip, server_port, id, 0, Ip.null) 
+      | _ -> key) cc in
   let rec c = {
       client_client = client_impl;
       client_kind = key;
       client_upload = None;
       client_source = s;
       client_ip = Ip.null;
+      client_country_code = cc;
       client_md4 = Md4.null;
       client_download = None;
       client_file_queue = [];
       client_tags = [];
       client_name = "";
-      client_last_asked_file = dummy_file;
       client_all_files = None;
       client_rating = 0;
       client_brand = Brand_unknown;
       client_brand_mod = Brand_mod_unknown;
+      client_osinfo = None;
       client_checked = false;
       client_connected = false;
-      client_downloaded = Int64.zero;
-      client_uploaded = Int64.zero;
+      client_total_downloaded = Int64.zero;
+      client_total_uploaded = Int64.zero;
+      client_session_downloaded = Int64.zero;
+      client_session_uploaded = Int64.zero;
       client_banned = false;
-      client_score = 0;
-      client_next_queue = 0;
       client_rank = 0;
       client_connect_time = 0;
       client_requests_received = 0;
       client_requests_sent = 0;
-      client_indirect_address = None;
       client_slot = SlotNotAsked;
       client_debug = Intset.mem s.DonkeySources.source_num !debug_clients;
       client_pending_messages = [];
@@ -643,6 +706,8 @@ let create_client key =
       client_sent_challenge = Int64.zero;
       client_public_key = None;
       client_sui_verified = None;
+      client_last_file_req_md4 = None;
+      client_osinfo_sent = false;
       } and    client_impl = {
       dummy_client_impl with
       impl_client_val = c;
@@ -654,19 +719,52 @@ let create_client key =
   CommonClient.new_client_with_num client_impl s.DonkeySources.source_num;
   H.add clients_by_kind c;
   clients_root := c :: !clients_root;
+  check_client_country_code c;
   c
 
-let new_client key =
+exception ClientFound of client
+let find_client_by_key key =
   try
-    H.find clients_by_kind { dummy_client with client_kind = key }
+    H.iter (fun c ->
+      if
+     (match c.client_kind with
+      | Indirect_address (server_ip, server_port, id, port, real_ip) ->
+          Indirect_address (server_ip, server_port, id, 0, Ip.null)
+      | _ -> c.client_kind) =
+     (match key with
+      | Indirect_address (server_ip, server_port, id, port, real_ip) ->
+          Indirect_address (server_ip, server_port, id, 0, Ip.null) 
+      | _ -> key) then
+        raise (ClientFound c)
+    ) clients_by_kind;
+    raise Not_found
+  with ClientFound c -> c
+
+let new_client key cc =
+  try
+    let c = find_client_by_key key in
+(* An indirect client without real_ip might have been created earlier.
+   If that client connected us later we have its real ip *)
+    (match key with
+    | Indirect_address (_,_,_,_,ip_real) ->
+        let old_ip =
+          match c.client_kind with
+          | Indirect_address (_,_,_,_,old_ip) ->
+              if old_ip = Ip.null then None else Some old_ip
+          | _ -> None
+        in
+        if ip_real <> Ip.null then c.client_kind <- key;
+        (match old_ip with
+        | Some old_ip ->
+            if old_ip <> ip_real then check_client_country_code c
+        | None -> if ip_real <> Ip.null then check_client_country_code c)
+    | _ -> ());
+    c
   with _ ->
-      create_client key
+      create_client key cc
 
 let create_client = ()
 
-let find_client_by_key key =
-  H.find clients_by_kind { dummy_client with client_kind = key }
-
 let client_type c =
   client_type (as_client c)
 
@@ -676,17 +774,8 @@ let set_client_type c t=
 let friend_add c =
   friend_add (as_client c)
 
-let string_of_client c =
-  Printf.sprintf "client[%d] %s(%s) %s" (client_num c)
-  c.client_name (brand_to_string c.client_brand)
-  (match c.client_kind with
-      Indirect_address (server_ip, server_port, ip, port, real_ip) ->
-        Printf.sprintf  " I[%s:%d]" (Ip.to_string real_ip) port;
-    | Direct_address (ip,port) ->
-        Printf.sprintf  " D[%s:%d]" (Ip.to_string ip) port;
-    | Invalid_address _ -> ""
-  )
-
+let string_of_server s =
+  Printf.sprintf "%s:%d" (Ip.to_string s.server_ip) s.server_port
 
 let set_client_name c name md4 =
   if name <> c.client_name || c.client_md4 <> md4 then begin
@@ -798,7 +887,6 @@ end;
   Printf.bprintf buf "  udp_servers_list: %d\n" (List.length !udp_servers_list);
   Printf.bprintf buf "  interesting_clients: %d\n" (List.length !interesting_clients);
   Printf.bprintf buf "  shared_files: %d\n" (List.length !shared_files);
-  Printf.bprintf buf "  new_shared_files: %d\n" (List.length !new_shared_files);
   Printf.bprintf buf "  servers_by_key: %d\n" (Hashtbl.length servers_by_key);
   Printf.bprintf buf "  banned_ips: %d\n" (Hashtbl.length banned_ips);
   Printf.bprintf buf "  old_requests: %d\n" (Hashtbl.length old_requests);
@@ -862,11 +950,6 @@ let last_connected_server () =
         [] -> raise Not_found
       | s :: _ -> s
 
-let all_servers () =
-  Hashtbl.fold (fun key s l ->
-      s :: l
-  ) servers_by_key []
-
 let string_of_file_state s =
   match  s with
   | FileDownloading -> "File Downloading"
@@ -929,21 +1012,20 @@ let save_join_queue c =
       with _ -> ()
     end
 
-let half_hour = 30 * 60
 let clean_join_queue_tables () =
   let current_time = last_time () in
 
   let list = Hashtbl2.to_list2 join_queue_by_md4 in
   Hashtbl.clear join_queue_by_md4;
   List.iter (fun (key, ((v,time) as e)) ->
-      if time + half_hour > current_time then
+      if time + Date.half_hour_in_secs > current_time then
         Hashtbl.add join_queue_by_md4 key e
   ) list;
 
   let list = Hashtbl2.to_list2 join_queue_by_id in
   Hashtbl.clear join_queue_by_id;
   List.iter (fun (key, ((v,time) as e)) ->
-      if time + half_hour > current_time then
+      if time + Date.half_hour_in_secs > current_time then
         Hashtbl.add join_queue_by_id key e
   ) list
 
@@ -980,18 +1062,18 @@ let _ =
       end
   )
 
-let server_accept_multiple_getsources s =
-  (s.server_flags land DonkeyProtoUdp.PingServerReplyUdp.multiple_getsources) <> 0
-
-let server_send_multiple_replies s =
-  (s.server_flags land DonkeyProtoUdp.PingServerReplyUdp.multiple_replies) <> 0
-
-let server_send_getsources2 s =
-  (s.server_flags land DonkeyProtoUdp.PingServerReplyUdp.getsources2) <> 0
-
 let full_client_identifier c =
     Printf.sprintf "%s (%s%s) '%s'"
-      (Ip.to_string c.client_ip)
-      (brand_to_string_short c.client_brand)
+      (match c.client_kind with
+          Indirect_address (server_ip, server_port, id, port, real_ip) ->
+            Printf.sprintf "%s:%d%s[lowID %Ld, server:%s:%d]"
+              (Ip.to_string real_ip) port
+              (match c.client_country_code with | None -> "" | Some cc -> Printf.sprintf "(%d)" cc)
+              id (Ip.to_string server_ip) server_port
+        | Direct_address (ip,port) ->
+            Printf.sprintf "%s:%d%s" (Ip.to_string ip) port
+              (match c.client_country_code with | None -> "" | Some cc -> Printf.sprintf "(%d)" cc)
+        | Invalid_address _ -> " invalid IP")
+      (GuiTypes.client_software_short (brand_to_string_short c.client_brand) c.client_osinfo)
       (if c.client_emule_proto.emule_release = "" then "" else " " ^ c.client_emule_proto.emule_release)
       (String.escaped c.client_name)
