@@ -284,8 +284,8 @@ let really_query_download filename size md4 location old_file absents user group
             CommonSwarming.set_absent swarmer absents
   end;
 
-  if !verbose then lprintf_nl (_b "Started new download, file %s, size %Ld, md4 %s")
-    (file_best_name file) size (Md4.to_string md4);
+  if !verbose then lprintf_nl (_b "Started new download, file %s, size %Ld, md4 %s, user:group %s:%s")
+    (file_best_name file) size (Md4.to_string md4) user.CommonTypes.user_name (CommonUserDb.user2_print_group group);
 
   DonkeyProtoOvernet.Overnet.recover_file file;
   DonkeyProtoKademlia.Kademlia.recover_file file;
@@ -1653,6 +1653,7 @@ let _ =
   network.op_network_reset <- (fun _ -> ());
 
   network.op_network_close_search <- (fun s -> ());
+  network.op_network_check_upload_slots <- (fun _ -> ());
   network.op_network_porttest_start <- (fun _ ->
     porttest_result := PorttestInProgress (last_time ());
     let module H = Http_client in
@@ -1888,9 +1889,9 @@ let _ =
 
 let _ =
   shared_ops.op_shared_state <- (fun f o ->
-    match CommonFile.file_state f with
+    match file_state f with
     | FileShared ->
-      (match file_shared f with
+      (match file_shared (as_file f) with
       | None -> "no file_shared info"
       | Some f ->
             let pre_share1_dir =
