@@ -78,33 +78,13 @@ let direct_client_sock_send emule_version sock m =
 
 let client_send c m =
   let emule_version = c.client_emule_proto in
-  if !verbose_msg_clients || c.client_debug then begin
-      lprintf_nl "Sent to client[%d] %s(%s) %s" (client_num c)
-        c.client_name (brand_to_string c.client_brand)
-      (match c.client_kind with
-          Indirect_address (server_ip, server_port, ip, port, real_ip) ->
-	    Printf.sprintf "[%s:%d]" (Ip.to_string (ip_of_id ip)) port
-        | Direct_address (ip,port) -> Printf.sprintf "[%s:%d]" (Ip.to_string ip) port
-        | Invalid_address _ -> ""
-      );
+  do_if_connected c.client_source.DonkeySources.source_sock (fun sock ->
+    if !verbose_msg_clients || c.client_debug then begin
+      lprintf_nl "Sent to client %s" (full_client_identifier c);
       DonkeyProtoClient.print m;
       lprint_newline ();
     end;
-  do_if_connected c.client_source.DonkeySources.source_sock (fun sock ->
-      direct_client_sock_send emule_version sock m)
-
-  (*
-let emule_send sock m =
-  let m = client_msg_to_string 0xc5 m in
-  (*
-  lprintf "Message to emule client:"; lprint_newline ();
-  LittleEndian.dump m;
-  lprint_newline ();
-  lprint_newline (); *)
-  write_string sock m
-    *)
-
-(* let client_msg_to_string m = client_msg_to_string 227 m *)
+    direct_client_sock_send emule_version sock m)
 
 let servers_send socks m =
   let m = server_msg_to_string m in
@@ -234,7 +214,8 @@ let tag_file file =
       if !verbose_share then lprintf_nl "tag_file: Sharing %s" name;
       name
     ))::
-  (int64_tag Field_Size file.file_file.impl_file_size) ::
+  (int64_tag Field_Size_Hi (Int64.shift_right_logical file.file_file.impl_file_size 32)) ::
+  (int64_tag Field_Size (Int64.logand file.file_file.impl_file_size 0xffffffffL)) ::
   (
     (match file.file_format with
         FormatNotComputed next_time when
@@ -339,13 +320,13 @@ let server_send_share compressed sock msg =
       str_int s 0 nfiles;
       let s = String.sub s 0 prev_len in
       if !verbose_share || !verbose then
-         lprintf_nl "Sending %d share(s) to server %s:%d%s"
-	   nfiles (Ip.to_string (peer_ip sock)) (peer_port sock)
+         lprintf_nl "Sending %d share%s to server %s:%d%s"
+	   nfiles (Printf2.print_plural_s nfiles) (Ip.to_string (peer_ip sock)) (peer_port sock)
 	   (if compressed then " (zlib)" else "");
       Buffer.reset buf;
       let s_c =
         if compressed then
-          Autoconf.zlib__compress_string s
+          Zlib.compress_string s
         else
           s
       in
