@@ -32,21 +32,37 @@ let lprintf_nl fmt =
 let lprintf_n fmt =
   lprintf2 log_prefix fmt
 
+exception Donkey_large_file
+
 type emule_proto = {
-    mutable emule_comments : int;
     mutable emule_version : int;
     mutable emule_release : string;
-    mutable emule_secident : int;
-    mutable emule_noviewshared : int;
-    mutable emule_supportpreview : int;
     mutable emule_osinfosupport : int;
-
-    mutable emule_compression : int;
-    mutable emule_sourceexchange : int;
-    mutable emule_multipacket : int;
-    mutable emule_extendedrequest : int;
     mutable emule_features : int;
+
+(* emule_miscoptions1 *)
+    mutable received_miscoptions1 : bool;
+    mutable emule_aich : int;
+    mutable emule_unicode : int;
     mutable emule_udpver : int;
+    mutable emule_compression : int;
+    mutable emule_secident : int;
+    mutable emule_sourceexchange : int;
+    mutable emule_extendedrequest : int;
+    mutable emule_comments : int;
+    mutable emule_peercache : int;
+    mutable emule_noviewshared : int;
+    mutable emule_multipacket : int;
+    mutable emule_supportpreview : int;
+
+(* emule_miscoptions2 *)
+    mutable received_miscoptions2 : bool;
+    mutable emule_require_crypt : int;
+    mutable emule_request_crypt : int;
+    mutable emule_support_crypt : int;
+    mutable emule_extmultipacket : int;
+    mutable emule_largefiles : int;
+    mutable emule_kad_version : int;
   }
 
 type emule_tag_name =
@@ -69,15 +85,6 @@ type request_record = {
   mutable nwarnings : int;
 }
 
-type client_score =
-  Client_not_connected
-| Client_has_file
-| Client_has_priority_file
-| Client_has_chunk
-| Client_has_priority_chunk
-| Client_has_upload
-| Client_has_priority_upload
-
 type reliability =
   Reliability_neutral
 | Reliability_reliable
@@ -100,6 +107,7 @@ type brand =
 | Brand_emuleplus
 | Brand_hydranode
 | Brand_verycd
+| Brand_imp
 
 let brand_list = [
    ( Brand_unknown   , "unknown"      , "unk" ) ;
@@ -118,6 +126,7 @@ let brand_list = [
    ( Brand_emuleplus , "ePlus"        , "eM+" ) ;
    ( Brand_hydranode , "Hydra"        , "Hyd" ) ;
    ( Brand_verycd    , "VeryCD"       , "VCD" ) ;
+   ( Brand_imp       , "IMPmule"      , "IMP" ) ;
   ]
 
 let brand_count = List.length brand_list
@@ -451,6 +460,7 @@ and server = (*[]*){
     mutable server_cid : Ip.t option;
     mutable server_port : int;
     mutable server_realport : int option; (* in case we connect through auxport; this is the true one *)
+    mutable server_country_code : int option;
     mutable server_sock : tcp_connection;
     mutable server_search_queries : CommonTypes.search Fifo.t;
     mutable server_users_queries : bool Fifo.t;
@@ -543,6 +553,7 @@ and client = {
     client_source : DonkeySources.source;
     mutable client_md4 : Md4.t;
     mutable client_ip : Ip.t;
+    mutable client_country_code : int option;
     mutable client_download : (file * CommonSwarming.uploader) option;
     mutable client_file_queue : (
       file * (* has displayed when connected *)
@@ -557,20 +568,19 @@ and client = {
     mutable client_checked : bool;
     mutable client_connected : bool;
 (* statistics *)
-    mutable client_downloaded : Int64.t;
-    mutable client_uploaded : Int64.t;
+    mutable client_session_downloaded : Int64.t;
+    mutable client_session_uploaded : Int64.t;
+    mutable client_total_downloaded : Int64.t;
+    mutable client_total_uploaded : Int64.t;
     mutable client_brand : brand;
     mutable client_brand_mod : brand_mod;
     mutable client_osinfo_sent : bool;
     mutable client_osinfo : string option;
     mutable client_banned : bool;
-    mutable client_score : int;
-    mutable client_next_queue : int;
     mutable client_rank : int;
     mutable client_connect_time : int;
     mutable client_requests_received : int;
     mutable client_requests_sent: int;
-    mutable client_indirect_address : (Ip.t * Ip.t * int) option;
     mutable client_slot : slot_status;
     mutable client_debug : bool;
     mutable client_pending_messages: string list;
@@ -602,6 +612,10 @@ and upload_info = {
     mutable up_pos : int64;
     mutable up_end_chunk : int64;
     mutable up_chunks : (int64 * int64) list;
+    (* zones sent but not yet received by other peer, oldest first *)
+    mutable up_flying_chunks : (int64 * int64) list; 
+    mutable up_current : int64;
+    mutable up_finish : bool;
     mutable up_waiting : bool;
   }
 
@@ -688,21 +702,41 @@ let server_state server =
   CommonServer.server_state (as_server server.server_server)
 
 let dummy_emule_proto = {
-    emule_comments = 0;
     emule_version = 0;
     emule_release = "";
-    emule_secident = 0;
-    emule_noviewshared = 0;
-    emule_supportpreview = 0;
     emule_osinfosupport = 0;
+    emule_features = 0;
 
+(* emule_miscoptions1 *)
+    received_miscoptions1 = false;
+    emule_aich = 0;
+    emule_unicode = 0;
+    emule_udpver = 0;
     emule_compression = 0; (* 1 *)
+    emule_secident = 0;
     emule_sourceexchange = 0; (* 3 *)
-    emule_multipacket = 0; (* 1 *)
     emule_extendedrequest = 0; (* 2 *)
-    emule_features = 0; (* 3 *)
-    emule_udpver = 0; (* 4 *)
+    emule_comments = 0;
+    emule_peercache = 0;
+    emule_noviewshared = 0;
+    emule_multipacket = 0;
+    emule_supportpreview = 0;
+
+(* emule_miscoptions2 *)
+    received_miscoptions2 = false;
+    emule_require_crypt = 0;
+    emule_request_crypt = 0;
+    emule_support_crypt = 0;
+    emule_extmultipacket = 0;
+    emule_largefiles = 0;
+    emule_kad_version = 0;
   }
 
 let emule_proto () =
   { dummy_emule_proto with emule_version = 0 }
+
+let old_max_emule_file_size = 4290048000L
+(* #define OLD_MAX_EMULE_FILE_SIZE 4290048000ui64  // (4294967295/PARTSIZE)*PARTSIZE = ~4GB *)
+
+let max_emule_file_size = 0x4000000000L
+(* #define MAX_EMULE_FILE_SIZE             0x4000000000ui64 // = 2^38 = 256GB *)
