@@ -108,7 +108,8 @@ end_pos !counter_pos b.len to_read;
             CommonSwarming.received up !counter_pos b.buf b.pos to_read_int;
             let new_downloaded = CommonSwarming.downloaded swarmer in
 
-	    c.client_downloaded <- c.client_downloaded ++ (new_downloaded -- old_downloaded);
+	    c.client_total_downloaded <- c.client_total_downloaded ++ (new_downloaded -- old_downloaded);
+	    c.client_session_downloaded <- c.client_session_downloaded ++ (new_downloaded -- old_downloaded);
             client_must_update (as_client c);
 
             if new_downloaded = file_size file then
@@ -278,13 +279,13 @@ let ftp_send_range_request c (x,y) sock d =
                 | "530 " ->
                     let reason = String.sub line 4 (slen - 4) in
                     if not (retry_530 reason) then begin
-                      pause_for_cause d.download_file "530" CommonUserDb.admin_user;
+                      pause_for_cause d.download_file "530" (CommonUserDb.admin_user ());
                     end else begin
                       c.client_reconnect <- true;
                     end;
                     disconnect_client c Closed_by_user;
                 | "550 " ->
-                    pause_for_cause d.download_file "550" CommonUserDb.admin_user;
+                    pause_for_cause d.download_file "550" (CommonUserDb.admin_user ());
                     disconnect_client c Closed_by_user;
                 | _ -> 
                     if !verbose then lprintf_nl "Unexpected line %s" line;
@@ -443,11 +444,11 @@ let ftp_check_size file url start_download_file =
                     | "530 " ->
                       let reason = String.sub line 4 (slen - 4) in
                       if not (retry_530 reason) then begin
-                        pause_for_cause file "530" CommonUserDb.admin_user;
+                        pause_for_cause file "530" (CommonUserDb.admin_user ());
                       end;
                       close sock Closed_by_user;
                     | "550 " ->
-                      pause_for_cause file "550" CommonUserDb.admin_user;
+                      pause_for_cause file "550" (CommonUserDb.admin_user ());
                       close sock Closed_by_user;
                     | _ -> 
                       if !verbose then lprintf_nl "Unexpected line %s" line;
@@ -463,7 +464,7 @@ let ftp_check_size file url start_download_file =
       TcpBufferedSocket.set_closer sock (fun _ _ -> ()
 (*        lprintf "Connection closed nread:%b\n" !nread; *)
       )
-  );
+  ) (fun _ -> ());
   ()
 
 (*************************************************************************)
