@@ -31,15 +31,15 @@ let chunk_min_size = ref 65000L
   
 let max_buffered = ref (Int64.of_int (1024 * 1024))
   
-let create_dir_mask = ref "755"
+let create_file_mode = ref 0o664
+let create_dir_mode = ref 0o755
 let verbose = ref false
-let max_cache_size = ref 50
+let max_cache_size_default = 50
+let max_cache_size = ref max_cache_size_default
   
 let mini (x: int) (y: int) =
   if x > y then y else x
     
-let rights = 0o664
-  
 let ro_flag =  [Unix.O_RDONLY]
 let rw_flag =  [Unix.O_RDWR]
 let rw_creat_flag =  [Unix.O_CREAT; Unix.O_RDWR]
@@ -117,11 +117,15 @@ module FDCache = struct
           | Some fd -> fd
           | None ->
 	      if !cache_size >= !max_cache_size then close_one ();
-	      let fd =
+	      let fd = Unix2.tryopen_umask 0 (fun _old_umask ->
+		(* disable umask restrictions temporarily; 
+		   rights on downloaded files should be dictated 
+		   by create_file_mode instead *)
 		try
 		  if t.writable then
                     Unix.openfile t.filename 
-		      (if creat then rw_creat_flag else rw_flag) rights
+		      (if creat then rw_creat_flag else rw_flag) 
+		      !create_file_mode
 		  else
 		    Unix.openfile t.filename ro_flag 0o400
 		with e ->
@@ -129,7 +133,7 @@ module FDCache = struct
 		    t.filename 
 		    (if t.writable then "rw" else "ro") 
                     (Printexc2.to_string e);
-		  raise e
+		  raise e)
 	      in
 		incr cache_size;
 (*              lprintf "local_force: opening %d\n" (Obj.magic fd_rw);  *)
@@ -204,16 +208,24 @@ module FDCache = struct
       raise e
 
     let owner t =
+      let module U = Unix.LargeFile in
       try
       check_destroyed t;
-      let s = Unix.fstat (local_force_fd t) in
-      let user = Unix.getpwuid s.Unix.st_uid in
-      let group = Unix.getgrgid s.Unix.st_gid in
+      let s =
+        try
+          U.fstat (local_force_fd t)
+        with
+ (* Unix.fstat is not supported on MinGW, Unix.stat is supported *)
+        | Invalid_argument _ -> U.stat t.filename
+      in
+      let user = Unix.getpwuid s.U.st_uid in
+      let group = Unix.getgrgid s.U.st_gid in
       user.Unix.pw_name, group.Unix.gr_name
-      with e ->
-      if !verbose then lprintf_nl "Exception in FDCache.owner %s: %s"
-        t.filename
-        (Printexc2.to_string e);
+      with
+      | Not_found -> "", ""
+      | e -> if !verbose then
+            lprintf_nl "Exception in FDCache.owner %s: %s"
+              t.filename (Printexc2.to_string e);
       raise e
 
     let mtime64 t =
@@ -254,9 +266,9 @@ module FDCache = struct
       check_destroyed t;
       close t;
       (let d = (Filename.dirname (Filename.concat f file)) in
-        Unix2.safe_mkdir d;
-        Unix2.chmod d (Misc.int_of_octal_string !create_dir_mask);
-	Unix2.can_write_to_directory d);
+       Unix2.safe_mkdir d;
+       Unix2.chmod d !create_dir_mode;
+       Unix2.can_write_to_directory d);
       (try
         Unix2.rename t.filename (Filename.concat f file);
       with
@@ -527,7 +539,7 @@ module MultiFile = struct
         let fd = FDCache.local_force_fd file.fd in
         f fd (chunk_begin -- file_begin)
       else
-	let temp_file = Filename.temp_file "chunk" ".tmp" in
+	let temp_file = Filename2.temp_file "chunk" ".tmp" in
 	let file_out = FDCache.create temp_file true in
 
 	try
@@ -834,7 +846,7 @@ module SparseFile = struct
       
       else
       
-	let temp_file = Filename.temp_file "chunk" ".tmp" in
+	let temp_file = Filename2.temp_file "chunk" ".tmp" in
 	let file_out = FDCache.create temp_file t.writable in
       
 	let rec iter pos index chunk_begin chunk_len =
@@ -1435,13 +1447,15 @@ let owner_fd t =
   | Destroyed -> "", ""
 
 let owner s =
-  try
     let old_fd_exists = fd_exists s in
     let fd = create_ro s in
-    let user,group = owner_fd fd in
-    if not old_fd_exists then close fd;
-    user,group
+  let user,pass =
+    try
+      owner_fd fd
   with _ -> "", ""
+  in
+  if not old_fd_exists then close fd;
+  user, pass
 
 let rename t f =
   flush_fd t;
@@ -1573,7 +1587,7 @@ The following functions have to be rewritten:
               iter tail
       
     and make_temp_file list = 
-      let temp_file = Filename.temp_file "chunk" ".tmp" in
+      let temp_file = Filename2.temp_file "chunk" ".tmp" in
       let file_out = create_rw temp_file in
         
       let rec fill pos chunk_begin chunk_len list =
@@ -1661,7 +1675,7 @@ type statfs = {
   f_files : int64;  (* total file nodes in file system *)
   f_ffree : int64;  (* free file nodes in fs *)
   f_fsid : unit;  (* See note in statfs(2) *)
-  f_fnamelen : int64; (* maximum length of filenames *)
+  f_fnamelen : int64; (* maximum length of filenames, maximum Sys.max_string_length *)
   f_basetype : string; (* type of filesystem - Solaris, (-1) on other systems, use f_type there *)
   f_frsize : int64;  (* Fundamental file system block size, (-1) if not provided by system *)
 }
@@ -1690,26 +1704,22 @@ let bsize dir =
 
 let blocks dir =
   try
-    let s = statfs dir in
-    Some s.f_blocks
+    Some (statfs dir).f_blocks
   with e -> None
 
 let bfree dir =
   try
-    let s = statfs dir in
-    Some s.f_bfree
+    Some (statfs dir).f_bfree
   with e -> None
 
 let bavail dir =
   try
-    let s = statfs dir in
-    Some s.f_bavail
+    Some (statfs dir).f_bavail
   with e -> None
 
 let fnamelen dir =
   try
-    let s = statfs dir in
-    Some s.f_fnamelen
+    Some (Int64.to_int (min (statfs dir).f_fnamelen (Int64.of_int Sys.max_string_length)))
   with e -> None
 
 let disktotal dir =
@@ -1826,6 +1836,12 @@ let filesystem dir =
 	     Printf.sprintf "unknown (%Ld)" s.f_type
   with e -> "not supported"
 
+let set_max_cache_size v =
+  max_cache_size := v;
+  while !FDCache.cache_size > !max_cache_size do FDCache.close_one () done
+
+let get_max_cache_size () = !max_cache_size
+
 let _ =
   Heap.add_memstat "Unix32" (fun level buf ->
       let counter = ref 0 in
