@@ -103,7 +103,6 @@ val peer_port : t -> int
   
 val stats :  Buffer.t -> t -> unit
 val buf_size : t -> int * int
-val can_fill : t -> bool
   
 (* val if_possible : bandwidth_controler -> int -> bool *)
   
@@ -144,7 +143,6 @@ val bind_address : Unix.inet_addr ref
 val ip_packet_size : int ref
 val mtu_packet_size : int ref
 val minimal_packet_size : int ref
-val packet_frame_size : int ref
   
 val get_rtimeout : t -> float * float
   
